package cheddar.aadl;

import org.osate.aadl2.instance.ComponentInstance;
import org.osate.aadl2.instance.SystemInstance;

import cheddar.adl.Address_Space;
import cheddar.adl.Core_Unit;
import cheddar.adl.Generic_Resource;
import cheddar.adl.Generic_Task;
import cheddar.osate2_plugin.Cheddar_Parameter;
import cheddar.osate2_plugin.Config;
import cheddar.osate2_plugin.Debug;


public class To_System {

	// Cheddar entities to produce
	//
	public cheddar.adl.System result;

	// Cheddar unique object identifier
	//
	Cheddar_Id_Generator id;


	public To_System() {
		id = new Cheddar_Id_Generator();
	}

	// Check if a system component is a SoC processor
	//
	public boolean is_SoC_Processing_Unit(ComponentInstance co) {
		Soc_Type sr = null;
			try {
			sr = AADLv3_Util.getSystemSocType(co);
			}
			catch (Exception e) {
			}
			if (sr != null) {
			if (sr == Soc_Type.SoC_Processing_Unit) {
				return true;
				}
			}
		return false;
	}

	// Check if a system component is a SoC interconnection
	//
	public boolean is_SoC_Interconnection(ComponentInstance co) {
		Soc_Type sr = null;
		try {
			sr = AADLv3_Util.getSystemSocType(co);
		} catch (Exception e) {
		}
		if (sr != null) {
			if (sr == Soc_Type.Soc_Interconnection_Unit) {
				return true;
			}
		}
		return false;
	}

	// Check if a system component is a SoC interconnection
	//
	public boolean is_SoC_Memory_Unit(ComponentInstance co) {
		Soc_Type sr = null;
		try {
			sr = AADLv3_Util.getSystemSocType(co);
		} catch (Exception e) {
		}
		if (sr != null) {
			if (sr == Soc_Type.SoC_Memory_Unit) {
				return true;
			}
		}
		return false;
	}

	// Entry point to transform AADL => Cheddar ADL
	//

	//
	// Uniprocessor : a uniprocessor model is composed of a AADL system component
	// with one or several AADL processors. Each AADL processor is a
	// Cheddar ADL uniprocessor component with a core prefixed by "core_"
	//
	// Global multicore : a global multicore model is composed of a AADL system component
	// with the System_Refinement property
	// with one or several AADL processors. Each AADL processor is a
	// Cheddar ADL core unit component and the AADL system component
	// is the Cheddar ADL multicore

	public String parse(SystemInstance si) {

		result = new cheddar.adl.System();

		// Read Cheddar system level properties to configure
		// Cheddar XML generation
		//
		Config.Debug = Properties_With_Default_Value.getDebug_Level(si);
		Debug.Put("Config.Debug_Level assigned with : " + Config.Debug);

		Config.Exported_Attribute_Time_Units = Properties_With_Default_Value.getExported_Attribute_Time_Units(si);
		Debug.Put("Config.Exported_Attribute_Time_Units assigned with : " + Config.Exported_Attribute_Time_Units);

		Config.Data_Port_To_Time_Triggered_Dependency = Properties_With_Default_Value.getIs_Data_Port_To_Time_Triggered_Dependency(si);
		Debug.Put("Config.Data_Port_To_Time_Triggered_Dependency assigned with : "
				+ Config.Data_Port_To_Time_Triggered_Dependency);

		Config.SporadicToPeriodic = Properties_With_Default_Value.getIs_Transform_Sporadic_To_Periodic(si);
		Debug.Put("Config.SporadicToPeriodic assigned with : " + Config.SporadicToPeriodic);

		Config.PoissonToPeriodic = Properties_With_Default_Value.getIs_Transform_Poisson_To_Periodic(si);
		Debug.Put("Config.PoissonToPeriodic assigned with : " + Config.PoissonToPeriodic);

		Config.Cheddar_Install_Folder = Properties_With_Default_Value.getCheddar_Install_Folder(si);
		Debug.Put("Config.Cheddar_Install_Folder assigned with : " + Config.Cheddar_Install_Folder);

		Config.Cheddar_Working_Folder = Properties_With_Default_Value.getCheddar_Working_Folder(si);
		Debug.Put("Config.Cheddar_Working_Folder assigned with : " + Config.Cheddar_Working_Folder);

		Cheddar_Parameter.Scheduling_Feasibility_Interval = Properties_With_Default_Value
				.getSchedulingFeasibilityInterval(si);
		Debug.Put("Cheddar_Parameter.Scheduling_Feasibility_Interval assigned with : "
				+ Cheddar_Parameter.Scheduling_Feasibility_Interval);

		Cheddar_Parameter.Scheduling_With_Offsets = Properties_With_Default_Value.getSchedulingWithOffsets(si);
		Debug.Put("Cheddar_Parameter.Scheduling_With_Offsets assigned with : "
				+ Cheddar_Parameter.Scheduling_With_Offsets);

		Cheddar_Parameter.Scheduling_With_Jitters = Properties_With_Default_Value.getSchedulingWithJitters(si);
		Debug.Put("Cheddar_Parameter.Scheduling_With_Jitters assigned with : "
				+ Cheddar_Parameter.Scheduling_With_Jitters);

		Cheddar_Parameter.Scheduling_With_CRPD = Properties_With_Default_Value.getSchedulingWithCRPD(si);
		Debug.Put("Cheddar_Parameter.Scheduling_With_CRPD assigned with : " + Cheddar_Parameter.Scheduling_With_CRPD);

		Cheddar_Parameter.Scheduling_With_Precendencies = Properties_With_Default_Value
				.getSchedulingWithPrecendencies(si);
		Debug.Put("Cheddar_Parameter.Scheduling_With_Precendencies assigned with : "
				+ Cheddar_Parameter.Scheduling_With_Precendencies);

		Cheddar_Parameter.Scheduling_With_Resources = Properties_With_Default_Value.getSchedulingWithResources(si);
		Debug.Put("Cheddar_Parameter.Scheduling_With_Resources assigned with : "
				+ Cheddar_Parameter.Scheduling_With_Resources);

		Cheddar_Parameter.Response_Time_From_Feasibility_Test = Properties_With_Default_Value
				.getResponseTimeFromFeasibilityTest(si);
		Debug.Put("Cheddar_Parameter.Response_Time_From_Feasibility_Test assigned with : "
				+ Cheddar_Parameter.Response_Time_From_Feasibility_Test);

		Cheddar_Parameter.Response_Time_From_Scheduling_Simulation = Properties_With_Default_Value
				.getResponseTimeFromSchedulingSimulation(si);
		Debug.Put("Cheddar_Parameter.Response_Time_From_Scheduling_Simulation assigned with : "
				+ Cheddar_Parameter.Response_Time_From_Scheduling_Simulation);



		// Create Cheddar ADL memory components
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case MEMORY:
				To_Memory.Create_Memory_Component(result, id, component);
				break;
			case SYSTEM:
				if (is_SoC_Memory_Unit(component)) {
				To_Memory.Create_Memory_Component(result, id, component);
				}
				break;
			default:
				break;
			}
		}

		// We parse all processor components
		//
		// 1) If a processor component is defined inside a system component which
		// is not a SoC processor => create a Cheddar core prefixed with "core_" and a
		// and a Cheddar uniprocessor
		//
		// 2) if a processor component is defined inside a SoC processor => create the
		// core, check if the multiprocessor exists and if not create it otherwise add
		// the core

		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case PROCESSOR:
				// Return enclosing system component
				ComponentInstance parent_system = component.getContainingComponentInstance();

				if (is_SoC_Processing_Unit(parent_system))
				{
					Core_Unit c = To_Core_Unit.Create_Unprefixed_Core_Unit(result, id, component);
					To_Processor.Create_MPSoC_Processor_Component(c, result, id, component, parent_system);
					}
				else
					{
					Core_Unit c = To_Core_Unit.Create_Prefixed_Core_Unit(result, id, component);
					To_Processor.Create_MonoCore_Processor(result, id, c, component);
				}
				break;
			default:
				break;
			}
		}


		// Create Cheddar ADL entities to model interconnection of processing/memory units
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case SYSTEM:
				if (is_SoC_Interconnection(component)) {
					To_Interconnection.Create_Interconnection_Component(result, id, component);
				}
				break;
			default:
				break;
			}
		}


		// Create Cheddar ADL address spaces
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case PROCESS:
				To_Address_Space.Create_Process_Component(result, id, component);
				break;
			case THREAD_GROUP:
				To_Address_Space.Create_Process_Component(result, id, component);
				break;
			default:
				break;
			}
		}

		// Create Cheddar ADL tasks and resources
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case THREAD:
				To_Task.Create_Thread_Component(result, id, component);
				break;
			case DATA:
				To_Resource.Create_Data_Component(result, id, component);
				break;
			default:
				break;
			}
		}


		// Create dependencies between Cheddar ADL instances
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case THREAD:
				To_Dependency.Thread_Component_Dependencies(result, id, component);
				break;
			default:
				break;
			}
		}


		// Depending on how bindings are expressed, we may have unbounded resources
		// or address spaces
		// For example, when threads and data are bounded individually, i.e. without
		// binding processes ... some Cheddar components may stay unbounded
		// Bellow we find such unbounded components and we try to fix it
		//
		// We may have address space components with unassigned "cpu_name"
		// Fix unassigned "cpu_name" attribute with the task binding
		//
		if (!result.address_spaces.isEmpty()) {
			for (Address_Space an_address_space : result.address_spaces) {
				if ( an_address_space.getCpuName().equals("aadl_component_is_not_bound")
						|| an_address_space.getCpuName().equals(""))
				{
					if (!result.tasks.isEmpty()) {
						for (Generic_Task a_task : result.tasks) {
							if (a_task.getAddressSpaceName().equals(an_address_space.getName())) {
								if (!a_task.getCpuName().equals("aadl_component_is_not_bound")) {
									an_address_space.setCpuName(a_task.getCpuName());
								}
							}
						}
					}
				}
			}
		}
		// We may have resource components components with unassigned "cpu_name"
		// Fix unassigned "cpu_name" attribute with the task binding
		//
		if (!result.resources.isEmpty()) {
			for (Generic_Resource a_resource : result.resources) {
				if (a_resource.getCpuName().equals("aadl_component_is_not_bound")) {
					Address_Space an_address_space = result.Search_Address_Space(a_resource.getAddressSpaceName());
					if (!an_address_space.getCpuName().equals("aadl_component_is_not_bound")) {
						a_resource.setCpuName(an_address_space.getCpuName());
					}
				}
			}
		}



		// Produce Cheddar XML V3 and return it
		//
		return result.xmlString();
	}




}
