package cheddar.osate2_plugin;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.osate.aadl2.Element;
import org.osate.aadl2.NamedElement;
import org.osate.aadl2.instance.ComponentInstance;
import org.osate.aadl2.instance.SystemInstance;
import org.osate.ui.dialogs.Dialog;

import cheddar.aadl.To_System;
import cheddar.core.XML_Pretty_Printer;

public class Response_Time_Plugin extends Abstract_Plugin {

	@Override
	public String getMarkerType() {
		return "cheddar.osate2_plugin.Response_Time_Plugin";
	}

	@Override
	protected String getActionName() {
		return "Response time with Cheddar 3.x";
	}

	@Override
	public boolean initializeAction(NamedElement obj) {
		setCSVLog("Response time with Cheddar 3.x", obj);
		return true;
	}


	@Override
	public final void doAaxlAction(final IProgressMonitor monitor, final Element obj) {

		if (!(obj instanceof ComponentInstance)) {
			Dialog.showError("Error", "The selected file is not at the right format");
			monitor.done();
			return;
		}

		SystemInstance si = ((ComponentInstance) obj).getSystemInstance();

		// Produce Cheddar model
		//
		String response;
		response = new To_System().parse(si);
		response = XML_Pretty_Printer.Pretty_Print(response);

		// Save Cheddar model in output file and run response_time_from_simulation
		//
		String file_info = WriteCheddarXML(si, response, si.getName());
		String msg = "Cheddar XML V3 model generated in the file :  " + file_info;
		Debug.Put(msg);
		Dialog.showInfo(
				"Instanciate Cheddar model and run compute response time with Cheddar 3.x, Cheddar model location:",
				msg);
		Launch l = new Launch();

		String response_time_method, scheduling_interval;
		if (
				(Cheddar_Parameter.Response_Time_From_Scheduling_Simulation
				&& Cheddar_Parameter.Response_Time_From_Feasibility_Test)
				||
				(!Cheddar_Parameter.Response_Time_From_Scheduling_Simulation
						&& !Cheddar_Parameter.Response_Time_From_Feasibility_Test)
		)
		{
			Dialog.showInfo("Error",
					"Response time with Cheddar 3.x : select either schedudling simulation or feasibility tests");
		}

		if (Cheddar_Parameter.Response_Time_From_Scheduling_Simulation) {
			response_time_method = " -s ";
			scheduling_interval = Cheddar_Parameter.Scheduling_Feasibility_Interval.toString();
		}
		else {
			response_time_method = " -f ";
			scheduling_interval = "";
		}

		l.executeCommand(
				Config.Cheddar_Install_Folder + "response_time " + response_time_method + " -i " + file_info + " -r "
						+ getCheddarOutputFolder(si) + "/response_time_" + si.getName() + ".txt  "
						+ scheduling_interval,
				Config.Cheddar_Environment,
				Config.Cheddar_Working_Folder);
		try {
			IResource resource = ResourcesPlugin.getWorkspace().getRoot();
			resource.refreshLocal(IResource.DEPTH_INFINITE, null);
		} catch (Exception ex) {
		}
	}




}
